searchData={"items":[{"type":"module","title":"Ecto.Adapters.FoundationDB","doc":"([Hex.pm](https://hex.pm/packages/ecto_foundationdb) | [GitHub](https://github.com/foundationdb-beam/ecto_foundationdb))\n\nAdapter module for FoundationDB.\n\nIt uses `:erlfdb` for communicating with the database.","ref":"Ecto.Adapters.FoundationDB.html"},{"type":"module","title":"Installation - Ecto.Adapters.FoundationDB","doc":"Install the latest stable release of FoundationDB from the\n[official FoundationDB Releases](https://github.com/apple/foundationdb/releases).\n\nThe `foundationdb-server` package is required on any system that will be running\na FoundationDB server instance. For example, it's common to\nrun the `foundationdb-server` on your development machine and on managed\ninstances running a FoundationDB cluster, but not for your stateless Elixir\napplication server in production.\n\n`foundationdb-clients` is always required.\n\nInclude `:ecto_foundationdb` in your list of dependencies in `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:ecto_foundationdb, \"~> 0.1\"}\n  ]\nend\n```","ref":"Ecto.Adapters.FoundationDB.html#module-installation"},{"type":"module","title":"Usage - Ecto.Adapters.FoundationDB","doc":"Define this module in your `MyApp` application:\n\n```elixir\ndefmodule MyApp.Repo do\n  use Ecto.Repo, otp_app: :my_app, adapter: Ecto.Adapters.FoundationDB\n\n  use EctoFoundationDB.Migrator\n  def migrations(), do: []\nend\n```\n\nEdit your `config.exs` file to include the following:\n\n```elixir\nconfig :my_app, MyApp.Repo,\n  cluster_file: \"/etc/foundationdb/fdb.cluster\"\n```\n\nIn your app's supervisor, you can start the repo as follows:\n\n```elixir\nchildren = [\n  ...\n  MyApp.Repo,\n  ...\n]\n\nSupervisor.start_link(...)\n```","ref":"Ecto.Adapters.FoundationDB.html#module-usage"},{"type":"module","title":"Tenants - Ecto.Adapters.FoundationDB","doc":"EctoFoundationDB requires the use of [FoundationDB Tenants](https://apple.github.io/foundationdb/tenants.html).\n\nEach tenant you create has a separate keyspace from all others, and a given FoundationDB\nTransaction is guaranteed to be isolated to a particular tenant's keyspace.\n\nYou'll use the Ecto `:prefix` option to specify the relevant tenant for each Ecto operation\nin your application.\n\nCreating a schema to be used in a tenant.\n\n```elixir\ndefmodule User do\n  use Ecto.Schema\n  @schema_context usetenant: true\n  @primary_key {:id, :binary_id, autogenerate: true}\n  schema \"users\" do\n    field(:name, :string)\n    field(:department, :string)\n    timestamps()\n  end\n  # ...\nend\n```\n\nSetting up a tenant.\n\n```elixir\nalias EctoFoundationDB.Tenant\n\ntenant = Tenant.open!(MyApp.Repo, \"some-org\")\n```\n\nInserting a new struct.\n\n```elixir\nuser = %User{name: \"John\", department: \"Engineering\"}\n       |> FoundationDB.usetenant(tenant)\n\nuser = MyApp.Repo.insert!(user)\n```\n\nQuerying for a struct using the primary key.\n\n```elixir\nMyApp.Repo.get!(User, user.id, prefix: tenant)\n```\n\nWhen a struct is retrieved from a tenant (using `:prefix`), that struct's metadata\nholds onto the tenant reference. This helps to protect your application from a\nstruct accidentally crossing tenant boundaries due to some unforeseen bug.","ref":"Ecto.Adapters.FoundationDB.html#module-tenants"},{"type":"module","title":"Indexes - Ecto.Adapters.FoundationDB","doc":"The implication of the [FoundationDB Layer Concept](https://apple.github.io/foundationdb/layer-concept.html)\nis that the manner in which data can be stored and accessed is the responsibility of the client\napplication. To achieve general purpose data storage and access patterns commonly desired\nin web applications, EctoFoundationDB provides support for indexes out of the box, and\nalso allows your application to define custom indexes.\n\nWhen you define a migration that calls `create index(User, [:department])`, a `Default` index is\ncreated on the `:department` field. The following two indexes are equivalent:\n\n```elixir\n# ...\ncreate index(User, [:department])\n# is equivalent to\ncreate index(User, [:department], options: [indexer: EctoFoundationDB.Indexer.Default]))]\n# ...\n```\n\nTimestamp indexes and multi-field indexes are supported:\n\n```elixir\ncreate index(User, [:inserted_at])\ncreate index(User, [:birthyear, :department, :birthdate])\n```\n\nIn fact, the index value can be any Elixir term. All types support Equal queries. However,\ncertain Ecto Schema types support Between queries. See the Data Types section for the list.\n\nFor example, When an index is created on timestamp-like fields, it is an effective time\nseries index. See the query examples below.\n\nSee the Migrations section for further details about managing indexes in your application.","ref":"Ecto.Adapters.FoundationDB.html#module-indexes"},{"type":"module","title":"Index Query Examples - Ecto.Adapters.FoundationDB","doc":"Retrieve all users in the Engineering department (with an Equal constraint):\n\n```elixir\niex> query = from(u in User, where: u.department == ^\"Engineering\")\niex> MyApp.Repo.all(query, prefix: tenant)\n```\n\nRetrieve all users with a birthyear from 1992 to 1994 (with a Between constraint):\n\n```elixir\niex> query = from(u in User,\n...>   where: u.birthyear >= ^1992 and u.birthyear <= ^1994\n...> )\niex> MyApp.Repo.all(query, prefix: tenant)\n```\n\nRetrieve all Engineers born in August 1992:\n\n```elixir\niex> query = from(u in User,\n...>   where: u.birthyear == ^1992 and\n...>          u.department == ^\"Engineering\" and\n...>          u.birthdate >= ^~D[1992-08-01] and u.birthdate < ^~D[1992-09-01]\n...> )\niex> MyApp.Repo.all(query, prefix: tenant)\n```\n\n**Order matters!**: When you create an index using multiple fields, the FDB key that stores the index will be extended with\nall the values in the order of your defined index fields. Because FoundationDB stores keys in a well-defined order,\nthe order of the fields in your index determines the Between queries you can perform.\n\nThere can be 0 or 1 Between clauses, and if one exists, it must correspond to the final constraint in the where clause when\ncompared against the order of the index fields.","ref":"Ecto.Adapters.FoundationDB.html#module-index-query-examples"},{"type":"module","title":"Custom Indexes - Ecto.Adapters.FoundationDB","doc":"As data is inserted, updated, deleted, and queried, the Indexer callbacks (via behaviour `EctoFoundationDB.Indexer`)\nare executed. Your Indexer module does all the work necessary to maintain the index within the transaction\nthat the data itself is being manipulated. Many different types of indexes are possible. You are in control!\n\nFor an example, please see the [MaxValue implementation](https://github.com/foundationdb-beam/ecto_foundationdb/blob/main/lib/ecto_foundationdb/indexer/max_value.ex)\nwhich is used internally by EctoFoundationDB to keep track of the maximum value of a field in a Schema.","ref":"Ecto.Adapters.FoundationDB.html#module-custom-indexes"},{"type":"module","title":"Transactions - Ecto.Adapters.FoundationDB","doc":"The Repo functions will automatically use FoundationDB transactions behind the scenes. EctoFoundationDB also\nexposes the Ecto.Repo transaction function, allowing you to group operations together with transactional isolation:\n\n```elixir\nMyApp.Repo.transaction(fn ->\n    # Ecto work\n  end, prefix: tenant)\n```\n\nIt also exposes a FoundationDB-specific transaction:\n\n```elixir\nalias Ecto.Adapters.FoundationDB\n\nFoundationDB.transactional(tenant,\n  fn tx ->\n    # :erlfdb work\n  end)\n```\n\nBoth of these calling conventions create a transaction on FDB. Typically you will use\n`Repo.transaction/1` when operating on your Ecto.Schema structs. If you wish\nto do anything using the lower-level `:erlfdb` API (rare), you will use\n`FoundationDB.transactional/2`.\n\nPlease read the [FoundationDB Developer Guid on transactions](https://apple.github.io/foundationdb/developer-guide.html#transaction-basics)\nfor more information about how to develop with transactions.\n\nIt's important to remember that even though the database gives ACID guarantees about the\nkeys updated in a transaction, the Elixir function passed into `Repo.transaction/1` or\n`FoundationDB.transactional/2` will be executed more than once when any conflicts\nare encountered. For this reason, your function must not have any side effects other than\nthe database operations. For example, do not publish any messages to other processes\n(such as `Phoenix.PubSub`) from within the transaction.\n\n```elixir\n# Do not do this!\nMyApp.Repo.transaction(fn ->\n  MyApp.Repo.insert(%User{name: \"John\"})\n  ...\n  Phoenix.PubSub.broadcast(\"my_topic\", \"new_user\") # Not safe! :(\nend, prefix: tenant)\n```\n\n```elixir\n# Instead, do this:\nMyApp.Repo.transaction(fn ->\n  MyApp.Repo.insert(%User{name: \"John\"})\n  ...\nend, prefix: tenant)\nPhoenix.PubSub.broadcast(\"my_topic\", \"new_user\") # Safe :)\n```","ref":"Ecto.Adapters.FoundationDB.html#module-transactions"},{"type":"module","title":"Migrations - Ecto.Adapters.FoundationDB","doc":"At first glance, EctoFoundationDB migrations may look similar to that of `:ecto_sql`,\nbut the actual execution of migrations and how your app must be configured are very\ndifferent, so please read this section in full.\n\nIf your app uses indexes on any of your schemas, you must include a module that implements\nthe `EctoFoundationDB.Migrator` behaviour. By default, this is assumed to be your Repo\nmodule, as we have demonstrated above.\n\nMigrations are only used for index management. There are no tables to add, drop, or modify.\nWhen you add a field to your Schema, any read requests on old objects will return `nil` in\nthat field.\n\nAs tenants are opened during your application runtime, migrations will be executed\nautomatically. This distributes the migration across a potentially long period of time,\nas migrations will not be executed unless the tenant is opened.\n\nThe Migrator is a module in your application runtime that provides the full list of\nordered migrations. These are the migrations that will be executed when a tenant is opened.\nIf you leave out a migration from the list, it will not be applied.\n\nThe result of `migrations/0` will change over time to include new migrations, like so:\n\n```elixir\ndefmodule MyApp.Repo do\n  # ...\n  def migrations() do\n    [\n      {0, MyApp.IndexUserByDepartment}\n      # At a later date, we my add a new index with a corresponding addition\n      # tho the migrations list:\n      #    {1, MyApp.IndexUserByRole}\n    ]\n  end\nend\n```\n\nEach migration is contained in a separate module, much like EctoSQL's. However, the operations\nto be carried out **must be returned as a list.** For example, the creation an index\nmay look like this:\n\n```elixir\ndefmodule MyApp.IndexUserByDepartment do\n  use EctoFoundationDB.Migration\n  def change() do\n    [\n      create index(User, [:department])\n    ]\n  end\nend\n```\n\nYour migrator and `MyApp. ` modules are part of your codebase. They must be\nincluded in your application release.\n\nMigrations can be completed in full at any time with a call to\n`EctoFoundationDB.Migrator.up_all/1`. Depending on how many tenants you have in\nyour database, and the size of the data for each tenant, this operation might take a long\ntime and make heavy use of system resources. It is safe to interrupt this operation, as\nmigrations are performed transactionally. However, if you interrupt a migration, the next\nattempt for that tenant may have a brief delay (~5 sec); the migrator must ensure that the\nprevious migration has indeed stopped executing.","ref":"Ecto.Adapters.FoundationDB.html#module-migrations"},{"type":"module","title":"Data Types - Ecto.Adapters.FoundationDB","doc":"EctoFoundationDB stores your struct's data using `:erlang.term_to_binary/1`, and\nretrieves it with `:erlang.binary_to_term/1`. As such, there is no data type\nconversion between Elixir types and Database Types. Any term you put in your\nstruct will be stored and later retrieved.\n\nData types are used by EctoFoundationDB for the creation and querying of indexes.\nCertain Ecto types are encoded into the FDB key, which allows you to formulate\nBetween queries on indexes of these types:\n\n - `:id`\n - `:binary_id`\n - `:integer`\n - `:float`\n - `:boolean`\n - `:string`\n - `:binary`\n - `:date`\n - `:time`\n - `:time_usec`\n - `:naive_datetime`\n - `:naive_datetime_usec`\n - `:utc_datetime`\n - `:utc_datetime_usec`\n\nSee Queries for more information.","ref":"Ecto.Adapters.FoundationDB.html#module-data-types"},{"type":"module","title":"Standard Options - Ecto.Adapters.FoundationDB","doc":"* `:cluster_file` - The path to the fdb.cluster file. The default is\n    `\"/etc/foundationdb/fdb.cluster\"`.\n  * `:migrator` - A module that implements the `EctoFoundationDB.Migrator`\n    behaviour. Required when using any indexes. Defaults to `nil`. When `nil`,\n    your Repo module is assumed to be the Migrator.","ref":"Ecto.Adapters.FoundationDB.html#module-standard-options"},{"type":"module","title":"Advanced Options - Ecto.Adapters.FoundationDB","doc":"* `:storage_id` - All tenants created by this adapter are prefixed with\n    this string. This allows multiple configurations to operate on the\n    same FoundationDB cluster indepedently. Defaults to\n    `\"Ecto.Adapters.FoundationDB\"`.\n  * `:open_db` - 0-arity function used for opening a reference to the\n     FoundationDB cluster. Defaults to `:erlfdb.open(cluster_file)`. When\n     using `EctoFoundationDB.Sandbox`, you should consider setting\n     this option to `Sandbox.open_db/0`.\n  * `:migration_step` - The maximum number of keys to process in a single transaction\n    when running migrations. Defaults to `1000`. If you use a number that is\n    too large, the FDB transactions run by the Migrator will fail.\n  * `:idx_cache` - When set to `:enabled`, the Ecto ets cache is used to store the\n    available indexes per tenant. This speeds up all database operations.\n    Defaults to `:enabled`.","ref":"Ecto.Adapters.FoundationDB.html#module-advanced-options"},{"type":"module","title":"Limitations and Caveats - Ecto.Adapters.FoundationDB","doc":"","ref":"Ecto.Adapters.FoundationDB.html#module-limitations-and-caveats"},{"type":"module","title":"FoundationDB - Ecto.Adapters.FoundationDB","doc":"Please read [FoundationDB Known Limitations](https://apple.github.io/foundationdb/known-limitations.html).","ref":"Ecto.Adapters.FoundationDB.html#module-foundationdb"},{"type":"module","title":"Layer - Ecto.Adapters.FoundationDB","doc":"EctoFoundationDB implements a specific Layer on the FoundationDB\nkey-value store. This Layer is intended to be generally useful, but you\nmay not find it suitable for your workloads. The Layer is documented in\ndetail at `EctoFoundationDB.Layer`. This project does not support\nplugging in other Layers.","ref":"Ecto.Adapters.FoundationDB.html#module-layer"},{"type":"module","title":"Tenants - Ecto.Adapters.FoundationDB","doc":"* The use of a tenant implies an ownership relationship between the tenant and the data.\n   It's up to you how you use this relationship in your application.\n\n * Failure to specify a tenant will result in a raised exception at runtime.","ref":"Ecto.Adapters.FoundationDB.html#module-tenants"},{"type":"module","title":"Migrations - Ecto.Adapters.FoundationDB","doc":"The following are not supported:\n\n* Renaming a table\n* Renaming a field\n* Dropping an index\n* Moving down in migration bersions (rollback)\n\nFinally, the Ecto Mix tasks are known to be unsupported:\n\n```elixir\n# These commands are not supported. Do not use them with :ecto_foundationdb!\n#    mix ecto.create\n#    mix ecto.drop\n#    mix ecto.migrate\n#    mix ecto.gen.migration\n#    mix ecto.rollback\n#    mix ecto.migrations\n```","ref":"Ecto.Adapters.FoundationDB.html#module-migrations"},{"type":"module","title":"Key and Value Size - Ecto.Adapters.FoundationDB","doc":"[FoundationDB has limits on key and value size](https://apple.github.io/foundationdb/known-limitations.html#large-keys-and-values)\nPlease be aware of these limitations as you develop. EctoFoundationDB doesn't make\nany attempt to protect you from these errors.","ref":"Ecto.Adapters.FoundationDB.html#module-key-and-value-size"},{"type":"module","title":"Ecto Queries - Ecto.Adapters.FoundationDB","doc":"You'll find that most queries outside of those detailed in the documentation fail with\nan `EctoFoundationDB.Exception.Unsupported` exception. The FoundationDB Layer concept\nprecludes complex queries from being executed within the database. Therefore, it only makes sense\nto implement a limited set of query types -- specifically those that do have optimized database\nquery semantics. All other filtering, aggregation, and grouping must be done by your Elixir code.\n\nIn other words, you'll be writing less SQL, and more Elixir.\n\nRefer to the integration tests for a collection of queries that is known to work.","ref":"Ecto.Adapters.FoundationDB.html#module-ecto-queries"},{"type":"module","title":"Other Ecto Features - Ecto.Adapters.FoundationDB","doc":"As you test the boundaries of EctoFoundationDB, you'll find that many other Ecto features just plain\ndon't work with this adapter. Ecto has a lot of features, and this adapter is in the early stage.\nFor any given unsupported feature, the underlying reason will be one of the following.\n\n1. It makes sense for EctoFoundationDB, but it just hasn't been implemented yet.\n2. The fundamental differences between FDB and SQL backends mean that there is\n   no practical reason to implement it in EctoFoundationDB.\n\nPlease feel free to request features, but also please be aware that your request\nmight be closed if it falls into that second category.","ref":"Ecto.Adapters.FoundationDB.html#module-other-ecto-features"},{"type":"function","title":"Ecto.Adapters.FoundationDB.db/1","doc":"","ref":"Ecto.Adapters.FoundationDB.html#db/1"},{"type":"function","title":"Ecto.Adapters.FoundationDB.transactional/2","doc":"Executes the given function in a transaction on the database.\n\nIf you provide an arity-0 function, your function will be executed in\na newly spawned process. This is to ensure that EctoFoundationDB can\nsafely manage the process dictionary.\n\nPlease be aware of the\n[limitations that FoundationDB](https://apple.github.io/foundationdb/developer-guide.html#transaction-basics)\nimposes on transactions.\n\nFor example, a transaction must complete\n[within 5 seconds](https://apple.github.io/foundationdb/developer-guide.html#long-running-transactions).","ref":"Ecto.Adapters.FoundationDB.html#transactional/2"},{"type":"function","title":"Ecto.Adapters.FoundationDB.usetenant/2","doc":"","ref":"Ecto.Adapters.FoundationDB.html#usetenant/2"},{"type":"module","title":"EctoFoundationDB.Database","doc":"See `Ecto.Adapters.FoundationDB`.","ref":"EctoFoundationDB.Database.html"},{"type":"type","title":"EctoFoundationDB.Database.t/0","doc":"","ref":"EctoFoundationDB.Database.html#t:t/0"},{"type":"module","title":"EctoFoundationDB.Index","doc":"Internal module only used if you are implementing a custom index.","ref":"EctoFoundationDB.Index.html"},{"type":"type","title":"EctoFoundationDB.Index.t/0","doc":"","ref":"EctoFoundationDB.Index.html#t:t/0"},{"type":"behaviour","title":"EctoFoundationDB.Indexer","doc":"Implement this behaviour to create a custom index.","ref":"EctoFoundationDB.Indexer.html"},{"type":"callback","title":"EctoFoundationDB.Indexer.clear/4","doc":"","ref":"EctoFoundationDB.Indexer.html#c:clear/4"},{"type":"function","title":"EctoFoundationDB.Indexer.clear/5","doc":"","ref":"EctoFoundationDB.Indexer.html#clear/5"},{"type":"function","title":"EctoFoundationDB.Indexer.create/5","doc":"","ref":"EctoFoundationDB.Indexer.html#create/5"},{"type":"callback","title":"EctoFoundationDB.Indexer.create/5","doc":"","ref":"EctoFoundationDB.Indexer.html#c:create/5"},{"type":"function","title":"EctoFoundationDB.Indexer.create_range/1","doc":"","ref":"EctoFoundationDB.Indexer.html#create_range/1"},{"type":"callback","title":"EctoFoundationDB.Indexer.create_range/1","doc":"","ref":"EctoFoundationDB.Indexer.html#c:create_range/1"},{"type":"function","title":"EctoFoundationDB.Indexer.range/3","doc":"","ref":"EctoFoundationDB.Indexer.html#range/3"},{"type":"callback","title":"EctoFoundationDB.Indexer.range/3","doc":"","ref":"EctoFoundationDB.Indexer.html#c:range/3"},{"type":"callback","title":"EctoFoundationDB.Indexer.set/4","doc":"","ref":"EctoFoundationDB.Indexer.html#c:set/4"},{"type":"function","title":"EctoFoundationDB.Indexer.set/5","doc":"","ref":"EctoFoundationDB.Indexer.html#set/5"},{"type":"function","title":"EctoFoundationDB.Indexer.unpack/3","doc":"","ref":"EctoFoundationDB.Indexer.html#unpack/3"},{"type":"callback","title":"EctoFoundationDB.Indexer.unpack/3","doc":"","ref":"EctoFoundationDB.Indexer.html#c:unpack/3"},{"type":"callback","title":"EctoFoundationDB.Indexer.update/4","doc":"","ref":"EctoFoundationDB.Indexer.html#c:update/4"},{"type":"function","title":"EctoFoundationDB.Indexer.update/5","doc":"","ref":"EctoFoundationDB.Indexer.html#update/5"},{"type":"module","title":"EctoFoundationDB.Layer","doc":"The Ecto FoundationDB Adapter implements a [Layer](https://apple.github.io/foundationdb/layer-concept.html)\non the underlying key-value store provided by FoundationDB. Via this layer, some\ncommon data access patterns are achieved. For those familiar with relational databases\nsuch as Postgres, these patterns will be familiar. However, there are many differences (for example SQL is\nnot supported), so this document seeks to describe the capabilities of the Ecto FoundationDB Layer in detail.","ref":"EctoFoundationDB.Layer.html"},{"type":"module","title":"Keyspace Design - EctoFoundationDB.Layer","doc":"All keys used by `:ecto_foundationdb` are encoded with [FoundationDB's Tuple Encoding](https://github.com/apple/foundationdb/blob/main/design/tuple.md)\nThe first element of the tuple is a string prefix that is intender to keep the `:ecto_foundationdb` keyspace\nseparate from other keys in the FoundationDB cluster.\n\nYour Schema data is stored with prefix \"\\xFD\".\n\nThe data associated with schema migrations is stored with prefix \"\\xFE\".\n\nAll values are either\n  * other keys (in the case of Default indexes) or\n  * Erlang term data encoded with `:erlang.term_to_binary/1`","ref":"EctoFoundationDB.Layer.html#module-keyspace-design"},{"type":"module","title":"Primary Write and Read - EctoFoundationDB.Layer","doc":"Your Ecto Schema has a primary key field, which is usually a string or an integer. This primary\nkey uniquely identifies an object of your schema within the tenant in which it lives.\n\n```elixir\ndefmodule EctoFoundationDB.Schemas.User do\n  use Ecto.Schema\n  @schema_context usetenant: true\n  schema \"users\" do\n    field(:name, :string)\n    field(:department, :string)\n    timestamps()\n  end\nend\n```\n\nIn this example, a User has an `:id` and a `:name`. Also notice that the User is defined with\n`usetenant: true` which provides a scope under which the User lives. For example, a typical tenant\nwould be the organization the User belongs to. Since the User is in this tenant, we do not need\nto provide an identifier for this organization on the User object itself.\n\n\"Primary Write\" refers to the insertion of the User struct into the FoundationDB key-value store\nunder a single key that uniquely identifies the User. This key includes the `:id` value.\n\nYour struct data is stored as a `Keyword` encoded with `:erlang.term_to_binary/1`.\n\nNote: The Primary Write can be skipped by providing the `write_primary: false` option on the `@schema_context`.\nSee below for more.","ref":"EctoFoundationDB.Layer.html#module-primary-write-and-read"},{"type":"module","title":"Default Indexes - EctoFoundationDB.Layer","doc":"When a Default index is created via a migration, the Ecto FoundationDB Adapter writes a set of\nkeys and values to facilitate lookups based on the indexed field.","ref":"EctoFoundationDB.Layer.html#module-default-indexes"},{"type":"module","title":"Advanced Options: `write_primary: false`, `mapped?: false` - EctoFoundationDB.Layer","doc":"If you choose to use `write_primary: false` on your schema, this skips the Primary Write. The consequence of this are as follows:\n\n 1. You'll want to make sure your index is created with an option `mapped?: false`. This ensures that the\n    struct data is written to the index keyspace.\n 2. Your index queries will now have performance characteristics similar to a primary key query. That is, you'll\n    be able to retrieve the struct data with a single `erlfdb:get_range/3`.\n 3. The data can **only** be managed by providing a query on the index. You will not be able to access the data\n    via the primary key.\n 4. If `write_primary: false`, then only one index can be created.","ref":"EctoFoundationDB.Layer.html#module-advanced-options-write_primary-false-mapped-false"},{"type":"behaviour","title":"EctoFoundationDB.Migrator","doc":"Implement this behaviour to define migrations for `Ecto.Adapters.FoundationDB`","ref":"EctoFoundationDB.Migrator.html"},{"type":"callback","title":"EctoFoundationDB.Migrator.migrations/0","doc":"","ref":"EctoFoundationDB.Migrator.html#c:migrations/0"},{"type":"function","title":"EctoFoundationDB.Migrator.up/3","doc":"","ref":"EctoFoundationDB.Migrator.html#up/3"},{"type":"function","title":"EctoFoundationDB.Migrator.up_all/2","doc":"","ref":"EctoFoundationDB.Migrator.html#up_all/2"},{"type":"module","title":"EctoFoundationDB.Options","doc":"Options for `Ecto.Adapters.FoundationDB`.","ref":"EctoFoundationDB.Options.html"},{"type":"function","title":"EctoFoundationDB.Options.get/2","doc":"","ref":"EctoFoundationDB.Options.html#get/2"},{"type":"type","title":"EctoFoundationDB.Options.option/0","doc":"","ref":"EctoFoundationDB.Options.html#t:option/0"},{"type":"type","title":"EctoFoundationDB.Options.t/0","doc":"","ref":"EctoFoundationDB.Options.html#t:t/0"},{"type":"module","title":"EctoFoundationDB.QueryPlan","doc":"Internal module only used if you are implementing a custom index.","ref":"EctoFoundationDB.QueryPlan.html"},{"type":"function","title":"EctoFoundationDB.QueryPlan.get/6","doc":"","ref":"EctoFoundationDB.QueryPlan.html#get/6"},{"type":"function","title":"EctoFoundationDB.QueryPlan.get_op/3","doc":"","ref":"EctoFoundationDB.QueryPlan.html#get_op/3"},{"type":"function","title":"EctoFoundationDB.QueryPlan.get_pinned_param/2","doc":"","ref":"EctoFoundationDB.QueryPlan.html#get_pinned_param/2"},{"type":"function","title":"EctoFoundationDB.QueryPlan.walk_ast/4","doc":"","ref":"EctoFoundationDB.QueryPlan.html#walk_ast/4"},{"type":"type","title":"EctoFoundationDB.QueryPlan.t/0","doc":"","ref":"EctoFoundationDB.QueryPlan.html#t:t/0"},{"type":"module","title":"EctoFoundationDB.Sandbox","doc":"A module for managing a sandbox FoundationDB cluster. This allows a developer to create\na space under which it should be safe to write tests.\n\nWhen using this module, it creates a directory in the project root called `.erlfdb`. It\nis safe to delete this directory when you no longer need it (e.g. after test execution)","ref":"EctoFoundationDB.Sandbox.html"},{"type":"function","title":"EctoFoundationDB.Sandbox.checkin/2","doc":"","ref":"EctoFoundationDB.Sandbox.html#checkin/2"},{"type":"function","title":"EctoFoundationDB.Sandbox.checkout/3","doc":"","ref":"EctoFoundationDB.Sandbox.html#checkout/3"},{"type":"function","title":"EctoFoundationDB.Sandbox.open_db/0","doc":"","ref":"EctoFoundationDB.Sandbox.html#open_db/0"},{"type":"module","title":"EctoFoundationDB.Tenant","doc":"This module allows the application to manage the creation and deletion of\ntenants within the FoundationDB database. All transactions require a tenant,\nso any application that uses the Ecto FoundationDB Adapter must use this module.","ref":"EctoFoundationDB.Tenant.html"},{"type":"function","title":"EctoFoundationDB.Tenant.clear/2","doc":"Clear all data for the given tenant. This cannot be undone.","ref":"EctoFoundationDB.Tenant.html#clear/2"},{"type":"function","title":"EctoFoundationDB.Tenant.clear/3","doc":"","ref":"EctoFoundationDB.Tenant.html#clear/3"},{"type":"function","title":"EctoFoundationDB.Tenant.clear_delete!/2","doc":"Clears data in a tenant and then deletes it. If the tenant doesn't exist, no-op.","ref":"EctoFoundationDB.Tenant.html#clear_delete!/2"},{"type":"function","title":"EctoFoundationDB.Tenant.create/2","doc":"Create a tenant in the database.","ref":"EctoFoundationDB.Tenant.html#create/2"},{"type":"function","title":"EctoFoundationDB.Tenant.create/3","doc":"","ref":"EctoFoundationDB.Tenant.html#create/3"},{"type":"function","title":"EctoFoundationDB.Tenant.db_open/3","doc":"","ref":"EctoFoundationDB.Tenant.html#db_open/3"},{"type":"function","title":"EctoFoundationDB.Tenant.db_open!/3","doc":"","ref":"EctoFoundationDB.Tenant.html#db_open!/3"},{"type":"function","title":"EctoFoundationDB.Tenant.delete/2","doc":"Deletes a tenant from the database permanently. The tenant must\nhave no data.","ref":"EctoFoundationDB.Tenant.html#delete/2"},{"type":"function","title":"EctoFoundationDB.Tenant.delete/3","doc":"","ref":"EctoFoundationDB.Tenant.html#delete/3"},{"type":"function","title":"EctoFoundationDB.Tenant.empty/3","doc":"","ref":"EctoFoundationDB.Tenant.html#empty/3"},{"type":"function","title":"EctoFoundationDB.Tenant.ensure_created/3","doc":"If the tenant doesn't exist, create it. Otherwise, no-op.","ref":"EctoFoundationDB.Tenant.html#ensure_created/3"},{"type":"function","title":"EctoFoundationDB.Tenant.exists?/2","doc":"Returns true if the tenant already exists in the database.","ref":"EctoFoundationDB.Tenant.html#exists?/2"},{"type":"function","title":"EctoFoundationDB.Tenant.exists?/3","doc":"Returns true if the tenant exists in the database. False otherwise.","ref":"EctoFoundationDB.Tenant.html#exists?/3"},{"type":"function","title":"EctoFoundationDB.Tenant.list/1","doc":"List all tenants in the database. Could be expensive.","ref":"EctoFoundationDB.Tenant.html#list/1"},{"type":"function","title":"EctoFoundationDB.Tenant.list/2","doc":"","ref":"EctoFoundationDB.Tenant.html#list/2"},{"type":"function","title":"EctoFoundationDB.Tenant.open/3","doc":"Open a tenant with a repo. With the result returned by this function, the caller can\ndo database operations on the tenant's portion of the key-value store.\n\nThe tenant must already exist.\n\nWhen opening tenants with a repo, all migrations are automatically performed. This\ncan cause open/2 to take a significant amount of time. Tenants can be kept open\nindefinitely, with any number of database transactions issued upon them.","ref":"EctoFoundationDB.Tenant.html#open/3"},{"type":"function","title":"EctoFoundationDB.Tenant.open!/3","doc":"Open a tenant. With the result returned by this function, the caller can\ndo database operations on the tenant's portion of the key-value store.\n\nIf the tenant does not exist, it is created.\n\nWhen opening tenants with a repo, all migrations are automatically performed. This\ncan cause open/2 to take a significant amount of time. Tenants can be kept open\nindefinitely, with any number of database transactions issued upon them.","ref":"EctoFoundationDB.Tenant.html#open!/3"},{"type":"function","title":"EctoFoundationDB.Tenant.open_empty!/3","doc":"Helper function to ensure the given tenant exists and then clear\nit of all data, and finally return an open handle. Useful in test code,\nbut in production, this would be dangerous.","ref":"EctoFoundationDB.Tenant.html#open_empty!/3"},{"type":"type","title":"EctoFoundationDB.Tenant.id/0","doc":"","ref":"EctoFoundationDB.Tenant.html#t:id/0"},{"type":"type","title":"EctoFoundationDB.Tenant.prefix/0","doc":"","ref":"EctoFoundationDB.Tenant.html#t:prefix/0"},{"type":"type","title":"EctoFoundationDB.Tenant.t/0","doc":"","ref":"EctoFoundationDB.Tenant.html#t:t/0"},{"type":"exception","title":"EctoFoundationDB.Exception.IncorrectTenancy","doc":"This exception is raised when there is a conflict in the tenant used.","ref":"EctoFoundationDB.Exception.IncorrectTenancy.html"},{"type":"exception","title":"EctoFoundationDB.Exception.Unsupported","doc":"This exception is raised when the application uses an Ecto feature that is not\nsupported by the FoundationDB Adapter.","ref":"EctoFoundationDB.Exception.Unsupported.html"},{"type":"extras","title":"EctoFoundationDB Guide","doc":"# EctoFoundationDB Guide\n\n```elixir\nMix.install([\n  {:ecto_foundationdb, \"~> 0.1\"}\n])\n```","ref":"guide.html"},{"type":"extras","title":"Setup - EctoFoundationDB Guide","doc":"Hello! This guide simulates what your experience might be when developing an application with EctoFoundationDB. Specifically, it focuses on the mechanism that EctoFoundationDB uses to create and manage indexes.\n\nIt assumes the reader is familiar with general Ecto features.\n\nBefore we get started, a couple of important points about executing these commands on your system.\n\n> If you received an error on the `Mix.install` setup, please make sure you have both `foundationdb-server` and `foundationdb-clients` packages installed on your system.\n\n> This LiveBook expects your system to have a running instance of FoundationDB, and it writes and deletes data from it. If your system's `/etc/foundationdb/fdb.cluster` is pointing to a real database, do not execute these commands!\n\nWith that out of the way, we'll start off with creating your Repo module.\n\n```elixir\ndefmodule MyApp.Repo do\n  use Ecto.Repo, otp_app: :my_app, adapter: Ecto.Adapters.FoundationDB\n\n  use EctoFoundationDB.Migrator\n\n  def migrations() do\n    [\n      # {0, IndexesMigration}\n    ]\n  end\nend\n```\n\nNotice that the line with `IndexesMigration` is commented out. We'll come back to this later.","ref":"guide.html#setup"},{"type":"extras","title":"Developing your app - EctoFoundationDB Guide","doc":"This next step simulates your app's startup. Normally, you would have a project defining `:my_app` and the Repo would be included in your supervision tree. In this Guide, we're starting the Repo as an isolated resource.\n\n```elixir\n{:ok, _} = Ecto.Adapters.FoundationDB.ensure_all_started(MyApp.Repo.config(), :temporary)\nMyApp.Repo.start_link(log: false)\n```\n\nNext, we define an `Ecto.Schema` for events that are coming from a temperature sensor. This is a pretty standard Schema module, but notice the `usetenant: true` option on `@schema_context`. This ensures that our `TemperatureEvent`s will be written to a Tenant in the database.\n\n```elixir\ndefmodule TemperatureEvent do\n  use Ecto.Schema\n\n  @schema_context usetenant: true\n  @primary_key {:id, :binary_id, autogenerate: true}\n\n  schema \"temperature_events\" do\n    field(:recorded_at, :naive_datetime_usec)\n    field(:kelvin, :float)\n    field(:site, :string)\n    timestamps()\n  end\nend\n```\n\nWe're going to create a module that will help us insert some `TemperatureEvents`.\n\n```elixir\ndefmodule Sensor do\n  alias Ecto.Adapters.FoundationDB\n\n  def record(n, tenant) do\n    MyApp.Repo.transaction(\n      fn ->\n        for _ <- 1..n, do: record(nil)\n      end,\n      prefix: tenant\n    )\n  end\n\n  def record(tenant) do\n    %TemperatureEvent{\n      site: \"surface\",\n      kelvin: 373.15 + :rand.normal(0, 5),\n      recorded_at: NaiveDateTime.utc_now()\n    }\n    |> FoundationDB.usetenant(tenant)\n    |> MyApp.Repo.insert!()\n  end\nend\n```\n\nNow, we create and open a new Tenant to store our `TemperatureEvents`.\n\n```elixir\nalias EctoFoundationDB.Tenant\n\ntenant = Tenant.open!(MyApp.Repo, \"experiment-42c\")\n```\n\nWe're ready to record an event from our temperature sensor! Feel free to Reevaluate this block several times. You'll record 4 new events each time.\n\n```elixir\nfor _ <- 1..4, do: Sensor.record(tenant)\n```\n\nWe can list all the events from the Tenant. This uses a single FoundationDB Transaction.\n\n```elixir\nMyApp.Repo.all(TemperatureEvent, prefix: tenant)\n```\n\nIf there's a large number of events, you can stream them instead of reading them all at once. This uses multiple FoundationDB Transactions.\n\n```elixir\nMyApp.Repo.stream(TemperatureEvent, prefix: tenant)\n|> Enum.to_list()\n|> length()\n```\n\nNext, we'd like to read all events from `\"surface\"`. If you're executing this LiveBook in order, you'll receive an exception on this step.\n\n```elixir\nimport Ecto.Query\n\nquery = from(e in TemperatureEvent, where: e.site == ^\"surface\")\nMyApp.Repo.all(query, prefix: tenant)\n```\n\nDid you get an exception? If so, scroll back up to the `defmodule MyApp.Repo` block in the Setup section, un-comment the line with `IndexesMigration`, and Reevaluate that block. Then come back and continue from here. You don't need to Reevaluate other blocks above this text.\n\n👋\n\nWelcome back! You've instructed the Repo to load a migration next time we open a Tenant. But we still need to define that Migration. The block below defines two indexes.\n\n```elixir\ndefmodule IndexesMigration do\n  use EctoFoundationDB.Migration\n\n  def change() do\n    [\n      create(index(TemperatureEvent, [:site])),\n      create(index(TemperatureEvent, [:recorded_at]))\n    ]\n  end\nend\n```\n\nNow, we re-open the Tenant. **Something very important happens here.**\n\nThis block simulates you restarting your app, and your client reconnecting. We'll just simply call `open!/2` again.\n\n```elixir\ntenant = Tenant.open!(MyApp.Repo, \"experiment-42c\")\n```\n\nGreat! If you made it to this step, then the Migration has executed automatically, and the indexes are ready to be used.","ref":"guide.html#developing-your-app"},{"type":"extras","title":"Querying your data - EctoFoundationDB Guide","doc":"This next block has the same query as the one that threw an exception earlier. This time, you should retrieve the expected events.\n\n```elixir\nimport Ecto.Query\nquery = from(e in TemperatureEvent, where: e.site == ^\"surface\")\nMyApp.Repo.all(query, prefix: tenant)\n```\n\nWe can also use the timestamp index that we created in a new query.\n\n```elixir\nnow = NaiveDateTime.utc_now()\npast = NaiveDateTime.add(now, -1200, :second)\n\nquery =\n  from(e in TemperatureEvent,\n    where: e.recorded_at >= ^past and e.recorded_at < ^now\n  )\n\nMyApp.Repo.all(query, prefix: tenant)\n```\n\nFinally, just for fun, let's insert 10,000 `TemperatureEvent`s!\n\n```elixir\nnum = 10000\nbatch = 100\n\n{t, :ok} =\n  :timer.tc(fn ->\n    Stream.duplicate(batch, div(num, batch))\n    |> Task.async_stream(\n      Sensor,\n      :record,\n      [tenant],\n      max_concurrency: System.schedulers_online() * 8,\n      ordered: false,\n      timeout: 30000\n    )\n    |> Stream.run()\n  end)\n\nIO.puts(\"Done in #{t / 1000} msec\")\n```","ref":"guide.html#querying-your-data"},{"type":"extras","title":"Cleaning up - EctoFoundationDB Guide","doc":"And if you'd like to tidy up, you can easily delete all the data.\n\n```elixir\n# Note: destructive!\nMyApp.Repo.delete_all(TemperatureEvent, prefix: tenant)\n```\n\n```elixir\n# Note: destructive!\nTenant.clear_delete!(MyApp.Repo, \"experiment-42c\")\n```","ref":"guide.html#cleaning-up"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}